# This stub file was automatically generated.

__all__ = [
    "QsolMass",
    "quettaM_sun",
    "quettaMsun",
    "RsolMass",
    "ronnaM_sun",
    "ronnaMsun",
    "YsolMass",
    "yottaM_sun",
    "yottaMsun",
    "ZsolMass",
    "zettaM_sun",
    "zettaMsun",
    "EsolMass",
    "exaM_sun",
    "exaMsun",
    "PsolMass",
    "petaM_sun",
    "petaMsun",
    "TsolMass",
    "teraM_sun",
    "teraMsun",
    "GsolMass",
    "gigaM_sun",
    "gigaMsun",
    "MsolMass",
    "megaM_sun",
    "megaMsun",
    "ksolMass",
    "kiloM_sun",
    "kiloMsun",
    "hsolMass",
    "hectoM_sun",
    "hectoMsun",
    "dasolMass",
    "dekaM_sun",
    "dekaMsun",
    "decaM_sun",
    "decaMsun",
    "dsolMass",
    "deciM_sun",
    "deciMsun",
    "csolMass",
    "centiM_sun",
    "centiMsun",
    "msolMass",
    "milliM_sun",
    "milliMsun",
    "usolMass",
    "μsolMass",
    "microM_sun",
    "microMsun",
    "nsolMass",
    "nanoM_sun",
    "nanoMsun",
    "psolMass",
    "picoM_sun",
    "picoMsun",
    "fsolMass",
    "femtoM_sun",
    "femtoMsun",
    "asolMass",
    "attoM_sun",
    "attoMsun",
    "zsolMass",
    "zeptoM_sun",
    "zeptoMsun",
    "ysolMass",
    "yoctoM_sun",
    "yoctoMsun",
    "rsolMass",
    "rontoM_sun",
    "rontoMsun",
    "qsolMass",
    "quectoM_sun",
    "quectoMsun",
    "QsolRad",
    "quettaR_sun",
    "quettaRsun",
    "RsolRad",
    "ronnaR_sun",
    "ronnaRsun",
    "YsolRad",
    "yottaR_sun",
    "yottaRsun",
    "ZsolRad",
    "zettaR_sun",
    "zettaRsun",
    "EsolRad",
    "exaR_sun",
    "exaRsun",
    "PsolRad",
    "petaR_sun",
    "petaRsun",
    "TsolRad",
    "teraR_sun",
    "teraRsun",
    "GsolRad",
    "gigaR_sun",
    "gigaRsun",
    "MsolRad",
    "megaR_sun",
    "megaRsun",
    "ksolRad",
    "kiloR_sun",
    "kiloRsun",
    "hsolRad",
    "hectoR_sun",
    "hectoRsun",
    "dasolRad",
    "dekaR_sun",
    "dekaRsun",
    "decaR_sun",
    "decaRsun",
    "dsolRad",
    "deciR_sun",
    "deciRsun",
    "csolRad",
    "centiR_sun",
    "centiRsun",
    "msolRad",
    "milliR_sun",
    "milliRsun",
    "usolRad",
    "μsolRad",
    "microR_sun",
    "microRsun",
    "nsolRad",
    "nanoR_sun",
    "nanoRsun",
    "psolRad",
    "picoR_sun",
    "picoRsun",
    "fsolRad",
    "femtoR_sun",
    "femtoRsun",
    "asolRad",
    "attoR_sun",
    "attoRsun",
    "zsolRad",
    "zeptoR_sun",
    "zeptoRsun",
    "ysolRad",
    "yoctoR_sun",
    "yoctoRsun",
    "rsolRad",
    "rontoR_sun",
    "rontoRsun",
    "qsolRad",
    "quectoR_sun",
    "quectoRsun",
    "QsolLum",
    "quettaL_sun",
    "quettaLsun",
    "RsolLum",
    "ronnaL_sun",
    "ronnaLsun",
    "YsolLum",
    "yottaL_sun",
    "yottaLsun",
    "ZsolLum",
    "zettaL_sun",
    "zettaLsun",
    "EsolLum",
    "exaL_sun",
    "exaLsun",
    "PsolLum",
    "petaL_sun",
    "petaLsun",
    "TsolLum",
    "teraL_sun",
    "teraLsun",
    "GsolLum",
    "gigaL_sun",
    "gigaLsun",
    "MsolLum",
    "megaL_sun",
    "megaLsun",
    "ksolLum",
    "kiloL_sun",
    "kiloLsun",
    "hsolLum",
    "hectoL_sun",
    "hectoLsun",
    "dasolLum",
    "dekaL_sun",
    "dekaLsun",
    "decaL_sun",
    "decaLsun",
    "dsolLum",
    "deciL_sun",
    "deciLsun",
    "csolLum",
    "centiL_sun",
    "centiLsun",
    "msolLum",
    "milliL_sun",
    "milliLsun",
    "usolLum",
    "μsolLum",
    "microL_sun",
    "microLsun",
    "nsolLum",
    "nanoL_sun",
    "nanoLsun",
    "psolLum",
    "picoL_sun",
    "picoLsun",
    "fsolLum",
    "femtoL_sun",
    "femtoLsun",
    "asolLum",
    "attoL_sun",
    "attoLsun",
    "zsolLum",
    "zeptoL_sun",
    "zeptoLsun",
    "ysolLum",
    "yoctoL_sun",
    "yoctoLsun",
    "rsolLum",
    "rontoL_sun",
    "rontoLsun",
    "qsolLum",
    "quectoL_sun",
    "quectoLsun",
]

from astropy.units.core import PrefixUnit

QsolMass: PrefixUnit
quettaM_sun: PrefixUnit
quettaMsun: PrefixUnit
RsolMass: PrefixUnit
ronnaM_sun: PrefixUnit
ronnaMsun: PrefixUnit
YsolMass: PrefixUnit
yottaM_sun: PrefixUnit
yottaMsun: PrefixUnit
ZsolMass: PrefixUnit
zettaM_sun: PrefixUnit
zettaMsun: PrefixUnit
EsolMass: PrefixUnit
exaM_sun: PrefixUnit
exaMsun: PrefixUnit
PsolMass: PrefixUnit
petaM_sun: PrefixUnit
petaMsun: PrefixUnit
TsolMass: PrefixUnit
teraM_sun: PrefixUnit
teraMsun: PrefixUnit
GsolMass: PrefixUnit
gigaM_sun: PrefixUnit
gigaMsun: PrefixUnit
MsolMass: PrefixUnit
megaM_sun: PrefixUnit
megaMsun: PrefixUnit
ksolMass: PrefixUnit
kiloM_sun: PrefixUnit
kiloMsun: PrefixUnit
hsolMass: PrefixUnit
hectoM_sun: PrefixUnit
hectoMsun: PrefixUnit
dasolMass: PrefixUnit
dekaM_sun: PrefixUnit
dekaMsun: PrefixUnit
decaM_sun: PrefixUnit
decaMsun: PrefixUnit
dsolMass: PrefixUnit
deciM_sun: PrefixUnit
deciMsun: PrefixUnit
csolMass: PrefixUnit
centiM_sun: PrefixUnit
centiMsun: PrefixUnit
msolMass: PrefixUnit
milliM_sun: PrefixUnit
milliMsun: PrefixUnit
usolMass: PrefixUnit
μsolMass: PrefixUnit
microM_sun: PrefixUnit
microMsun: PrefixUnit
nsolMass: PrefixUnit
nanoM_sun: PrefixUnit
nanoMsun: PrefixUnit
psolMass: PrefixUnit
picoM_sun: PrefixUnit
picoMsun: PrefixUnit
fsolMass: PrefixUnit
femtoM_sun: PrefixUnit
femtoMsun: PrefixUnit
asolMass: PrefixUnit
attoM_sun: PrefixUnit
attoMsun: PrefixUnit
zsolMass: PrefixUnit
zeptoM_sun: PrefixUnit
zeptoMsun: PrefixUnit
ysolMass: PrefixUnit
yoctoM_sun: PrefixUnit
yoctoMsun: PrefixUnit
rsolMass: PrefixUnit
rontoM_sun: PrefixUnit
rontoMsun: PrefixUnit
qsolMass: PrefixUnit
quectoM_sun: PrefixUnit
quectoMsun: PrefixUnit
QsolRad: PrefixUnit
quettaR_sun: PrefixUnit
quettaRsun: PrefixUnit
RsolRad: PrefixUnit
ronnaR_sun: PrefixUnit
ronnaRsun: PrefixUnit
YsolRad: PrefixUnit
yottaR_sun: PrefixUnit
yottaRsun: PrefixUnit
ZsolRad: PrefixUnit
zettaR_sun: PrefixUnit
zettaRsun: PrefixUnit
EsolRad: PrefixUnit
exaR_sun: PrefixUnit
exaRsun: PrefixUnit
PsolRad: PrefixUnit
petaR_sun: PrefixUnit
petaRsun: PrefixUnit
TsolRad: PrefixUnit
teraR_sun: PrefixUnit
teraRsun: PrefixUnit
GsolRad: PrefixUnit
gigaR_sun: PrefixUnit
gigaRsun: PrefixUnit
MsolRad: PrefixUnit
megaR_sun: PrefixUnit
megaRsun: PrefixUnit
ksolRad: PrefixUnit
kiloR_sun: PrefixUnit
kiloRsun: PrefixUnit
hsolRad: PrefixUnit
hectoR_sun: PrefixUnit
hectoRsun: PrefixUnit
dasolRad: PrefixUnit
dekaR_sun: PrefixUnit
dekaRsun: PrefixUnit
decaR_sun: PrefixUnit
decaRsun: PrefixUnit
dsolRad: PrefixUnit
deciR_sun: PrefixUnit
deciRsun: PrefixUnit
csolRad: PrefixUnit
centiR_sun: PrefixUnit
centiRsun: PrefixUnit
msolRad: PrefixUnit
milliR_sun: PrefixUnit
milliRsun: PrefixUnit
usolRad: PrefixUnit
μsolRad: PrefixUnit
microR_sun: PrefixUnit
microRsun: PrefixUnit
nsolRad: PrefixUnit
nanoR_sun: PrefixUnit
nanoRsun: PrefixUnit
psolRad: PrefixUnit
picoR_sun: PrefixUnit
picoRsun: PrefixUnit
fsolRad: PrefixUnit
femtoR_sun: PrefixUnit
femtoRsun: PrefixUnit
asolRad: PrefixUnit
attoR_sun: PrefixUnit
attoRsun: PrefixUnit
zsolRad: PrefixUnit
zeptoR_sun: PrefixUnit
zeptoRsun: PrefixUnit
ysolRad: PrefixUnit
yoctoR_sun: PrefixUnit
yoctoRsun: PrefixUnit
rsolRad: PrefixUnit
rontoR_sun: PrefixUnit
rontoRsun: PrefixUnit
qsolRad: PrefixUnit
quectoR_sun: PrefixUnit
quectoRsun: PrefixUnit
QsolLum: PrefixUnit
quettaL_sun: PrefixUnit
quettaLsun: PrefixUnit
RsolLum: PrefixUnit
ronnaL_sun: PrefixUnit
ronnaLsun: PrefixUnit
YsolLum: PrefixUnit
yottaL_sun: PrefixUnit
yottaLsun: PrefixUnit
ZsolLum: PrefixUnit
zettaL_sun: PrefixUnit
zettaLsun: PrefixUnit
EsolLum: PrefixUnit
exaL_sun: PrefixUnit
exaLsun: PrefixUnit
PsolLum: PrefixUnit
petaL_sun: PrefixUnit
petaLsun: PrefixUnit
TsolLum: PrefixUnit
teraL_sun: PrefixUnit
teraLsun: PrefixUnit
GsolLum: PrefixUnit
gigaL_sun: PrefixUnit
gigaLsun: PrefixUnit
MsolLum: PrefixUnit
megaL_sun: PrefixUnit
megaLsun: PrefixUnit
ksolLum: PrefixUnit
kiloL_sun: PrefixUnit
kiloLsun: PrefixUnit
hsolLum: PrefixUnit
hectoL_sun: PrefixUnit
hectoLsun: PrefixUnit
dasolLum: PrefixUnit
dekaL_sun: PrefixUnit
dekaLsun: PrefixUnit
decaL_sun: PrefixUnit
decaLsun: PrefixUnit
dsolLum: PrefixUnit
deciL_sun: PrefixUnit
deciLsun: PrefixUnit
csolLum: PrefixUnit
centiL_sun: PrefixUnit
centiLsun: PrefixUnit
msolLum: PrefixUnit
milliL_sun: PrefixUnit
milliLsun: PrefixUnit
usolLum: PrefixUnit
μsolLum: PrefixUnit
microL_sun: PrefixUnit
microLsun: PrefixUnit
nsolLum: PrefixUnit
nanoL_sun: PrefixUnit
nanoLsun: PrefixUnit
psolLum: PrefixUnit
picoL_sun: PrefixUnit
picoLsun: PrefixUnit
fsolLum: PrefixUnit
femtoL_sun: PrefixUnit
femtoLsun: PrefixUnit
asolLum: PrefixUnit
attoL_sun: PrefixUnit
attoLsun: PrefixUnit
zsolLum: PrefixUnit
zeptoL_sun: PrefixUnit
zeptoLsun: PrefixUnit
ysolLum: PrefixUnit
yoctoL_sun: PrefixUnit
yoctoLsun: PrefixUnit
rsolLum: PrefixUnit
rontoL_sun: PrefixUnit
rontoLsun: PrefixUnit
qsolLum: PrefixUnit
quectoL_sun: PrefixUnit
quectoLsun: PrefixUnit
