# This stub file was automatically generated.

__all__ = [
    "enable",
    "inch",
    "ft",
    "foot",
    "yd",
    "yard",
    "mi",
    "mile",
    "mil",
    "thou",
    "nmi",
    "nauticalmile",
    "NM",
    "fur",
    "furlong",
    "ac",
    "acre",
    "gallon",
    "quart",
    "pint",
    "cup",
    "foz",
    "fluid_oz",
    "fluid_ounce",
    "tbsp",
    "tablespoon",
    "tsp",
    "teaspoon",
    "oz",
    "ounce",
    "lb",
    "lbm",
    "pound",
    "st",
    "stone",
    "ton",
    "slug",
    "kn",
    "kt",
    "knot",
    "NMPH",
    "lbf",
    "kip",
    "kilopound",
    "BTU",
    "btu",
    "cal",
    "calorie",
    "kcal",
    "Cal",
    "Calorie",
    "kilocal",
    "kilocalorie",
    "psi",
    "hp",
    "horsepower",
    "deg_F",
    "Fahrenheit",
    "deg_R",
    "Rankine",
]

from astropy.units.core import IrreducibleUnit
from astropy.units.core import Unit
from astropy.units.core import _UnitContext

inch: Unit
ft: Unit
foot: Unit
yd: Unit
yard: Unit
mi: Unit
mile: Unit
mil: Unit
thou: Unit
nmi: Unit
nauticalmile: Unit
NM: Unit
fur: Unit
furlong: Unit
ac: Unit
acre: Unit
gallon: Unit
quart: Unit
pint: Unit
cup: Unit
foz: Unit
fluid_oz: Unit
fluid_ounce: Unit
tbsp: Unit
tablespoon: Unit
tsp: Unit
teaspoon: Unit
oz: Unit
ounce: Unit
lb: Unit
lbm: Unit
pound: Unit
st: Unit
stone: Unit
ton: Unit
slug: Unit
kn: Unit
kt: Unit
knot: Unit
NMPH: Unit
lbf: Unit
kip: Unit
kilopound: Unit
BTU: Unit
btu: Unit
cal: Unit
calorie: Unit
kcal: Unit
Cal: Unit
Calorie: Unit
kilocal: Unit
kilocalorie: Unit
psi: Unit
hp: Unit
horsepower: Unit
deg_F: IrreducibleUnit
Fahrenheit: IrreducibleUnit
deg_R: Unit
Rankine: Unit

def enable() -> _UnitContext: ...
