# This stub file was automatically generated.

__all__ = [
    "dex",
    "dB",
    "decibel",
    "mag",
    "Qmag",
    "Rmag",
    "Ymag",
    "Zmag",
    "Emag",
    "Pmag",
    "Tmag",
    "Gmag",
    "Mmag",
    "kmag",
    "hmag",
    "damag",
    "dmag",
    "cmag",
    "mmag",
    "umag",
    "μmag",
    "nmag",
    "pmag",
    "fmag",
    "amag",
    "zmag",
    "ymag",
    "rmag",
    "qmag",
    "STmag",
    "ABmag",
    "M_bol",
    "m_bol",
]

from astropy.units.core import PrefixUnit
from astropy.units.function.logarithmic import MagUnit
from astropy.units.function.mixin import IrreducibleFunctionUnit
from astropy.units.function.mixin import RegularFunctionUnit

dex: IrreducibleFunctionUnit
dB: RegularFunctionUnit
decibel: RegularFunctionUnit
mag: RegularFunctionUnit
Qmag: PrefixUnit
Rmag: PrefixUnit
Ymag: PrefixUnit
Zmag: PrefixUnit
Emag: PrefixUnit
Pmag: PrefixUnit
Tmag: PrefixUnit
Gmag: PrefixUnit
Mmag: PrefixUnit
kmag: PrefixUnit
hmag: PrefixUnit
damag: PrefixUnit
dmag: PrefixUnit
cmag: PrefixUnit
mmag: PrefixUnit
umag: PrefixUnit
μmag: PrefixUnit
nmag: PrefixUnit
pmag: PrefixUnit
fmag: PrefixUnit
amag: PrefixUnit
zmag: PrefixUnit
ymag: PrefixUnit
rmag: PrefixUnit
qmag: PrefixUnit
STmag: MagUnit
ABmag: MagUnit
M_bol: MagUnit
m_bol: MagUnit
