# This stub file was automatically generated.

__all__ = [
    "Bol",
    "L_bol",
    "bol",
    "f_bol",
    "AB",
    "ABflux",
    "ST",
    "STflux",
    "mgy",
    "maggy",
    "nmgy",
    "nanomaggy",
]

from astropy.units.core import IrreducibleUnit
from astropy.units.core import PrefixUnit
from astropy.units.core import Unit

Bol: Unit
L_bol: Unit
bol: Unit
f_bol: Unit
AB: Unit
ABflux: Unit
ST: Unit
STflux: Unit
mgy: IrreducibleUnit
maggy: IrreducibleUnit
nmgy: PrefixUnit
nanomaggy: PrefixUnit
